
""" PySpiral.py
Commands for Python Spirals
"""

import sys, os, re, math, wx

from CADdy.AddInServer import *
from CADdy.Application import GetApplic
from CADdy.Utility import Cast
from CADdy.Utility.Model import AddToParent
from win32com.client import constants

###############################################################################
CLSID='{AC6A1934-FCC8-4CAE-9834-0BD9FE878A45}'
PROGID= 'CADdyAddIn.PySpiral'

###############################################################################
from pywin.debugger import *
# set_trace()

###############################################################################

###############################################################################
##  Dialog
##
class ParameterSpiral:

    # Zusammenstellung der steuernden Parameter
    def __init__(self, spiralType, a, k, w1, w2, wStep):

        self.spiralType = spiralType # 'log' or 'arc'
        self.a          = a
        self.k          = k
        self.w1         = w1
        self.w2         = w2
        self.wStep      = wStep
        self.valid      = False

class FrmSpiral(wx.Frame):

    def __init__(self, *args, **kwds):
        # begin wxGlade: FrmSpiral.__init__
        kwds["style"] = wx.DEFAULT_FRAME_STYLE
        wx.Frame.__init__(self, *args, **kwds)
        self.panel = wx.Panel(self, -1)
        
        self.label_1 = wx.StaticText(self.panel, -1, "Spiraltyp:")
        #self.text_ctrl_1 = wx.TextCtrl(self.panel, -1, "")
        self.text_ctrl_1 = wx.ComboBox(self.panel, -1, choices=["log","arc"])

        self.label_2 = wx.StaticText(self.panel, -1, "a:")
        self.text_ctrl_2 = wx.TextCtrl(self.panel, -1, "")

        self.label_3 = wx.StaticText(self.panel, -1, "k:")
        self.text_ctrl_3 = wx.TextCtrl(self.panel, -1, "")

        self.label_4 = wx.StaticText(self.panel, -1, "Starwinkel:")
        self.text_ctrl_4 = wx.TextCtrl(self.panel, -1, "")

        self.label_5 = wx.StaticText(self.panel, -1, "Endwinkel:")
        self.text_ctrl_5 = wx.TextCtrl(self.panel, -1, "")

        self.label_6 = wx.StaticText(self.panel, -1, "Winkelaufloesung:")
        self.text_ctrl_6 = wx.TextCtrl(self.panel, -1, "")

        self.button_1 = wx.Button(self.panel, -1, "OK")
        self.button_2 = wx.Button(self.panel, -1, "Abbruch")

        self.Bind(wx.EVT_BUTTON, self.OnClose, self.button_1)
        self.Bind(wx.EVT_BUTTON, self.OnAbort, self.button_2)
        self.Bind(wx.EVT_CLOSE,  self.OnCloseWindow)

        self.__set_properties()
        self.__do_layout()
        # end wxGlade

    def DataToControls(self):
        self.text_ctrl_1.Value = str(self.param.spiralType)
        self.text_ctrl_2.Value = str(self.param.a)
        self.text_ctrl_3.Value = str(self.param.k)
        self.text_ctrl_4.Value = str(self.param.w1)
        self.text_ctrl_5.Value = str(self.param.w2)
        self.text_ctrl_6.Value = str(self.param.wStep)

    def ControlsToData(self):
        self.param.spiralType = self.text_ctrl_1.Value
        self.param.a          = float(self.text_ctrl_2.Value)
        self.param.k          = float(self.text_ctrl_3.Value)
        self.param.w1         = float(self.text_ctrl_4.Value)
        self.param.w2         = float(self.text_ctrl_5.Value)
        self.param.wStep      = float(self.text_ctrl_6.Value)

    def OnClose(self, event):
        self.ControlsToData()
        self.param.valid = True
        self.Close(True)

    def OnAbort(self, event):
        self.Close(True)

    def OnCloseWindow(self, event):
        self.Destroy()

    def __set_properties(self):
        # begin wxGlade: FrmSpiral.__set_properties
        self.SetTitle("Parameter Spirale")
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: FrmSpiral.__do_layout
        grid_sizer_1 = wx.GridSizer(1, 1, 0, 0)
        grid_sizer_1.Add(self.panel, 0, 0, 0)
        grid_sizer_2 = wx.GridSizer(5, 2, 0, 0)
        grid_sizer_2.Add(self.label_1, 0, 0, 0)
        grid_sizer_2.Add(self.text_ctrl_1, 0, 0, 0)
        grid_sizer_2.Add(self.label_2, 0, 0, 0)
        grid_sizer_2.Add(self.text_ctrl_2, 0, 0, 0)
        grid_sizer_2.Add(self.label_3, 0, 0, 0)
        grid_sizer_2.Add(self.text_ctrl_3, 0, 0, 0)
        grid_sizer_2.Add(self.label_4, 0, 0, 0)
        grid_sizer_2.Add(self.text_ctrl_4, 0, 0, 0)
        grid_sizer_2.Add(self.label_5, 0, 0, 0)
        grid_sizer_2.Add(self.text_ctrl_5, 0, 0, 0)
        grid_sizer_2.Add(self.label_6, 0, 0, 0)
        grid_sizer_2.Add(self.text_ctrl_6, 0, 0, 0)
        grid_sizer_2.Add(self.button_1, 0, 0, 0)
        grid_sizer_2.Add(self.button_2, 0, 0, 0)
        self.panel.SetSizer(grid_sizer_2)
        grid_sizer_2.Fit(self.panel)
        self.SetSizer(grid_sizer_1)
        grid_sizer_1.Fit(self)
        self.Layout()
        # end wxGlade

def ParameterDialog(param):
    app = wx.PySimpleApp(0)
    wx.InitAllImageHandlers()
    frame_1 = FrmSpiral(None, -1, "")
    frame_1.SetWindowStyle(frame_1.GetWindowStyle() | wx.STAY_ON_TOP)
    app.SetTopWindow(frame_1)
    frame_1.param = param
    frame_1.DataToControls()
    frame_1.Show()
    app.MainLoop()

    return param

###############################################################################
##  Helper
##
def getPoint(param,x,y,angle,w):
    types    = GetApplic().DataTypes()

    pos = types.CADdy2DPosition()
    aRadian = angle * math.atan(1.0) / 45.0;
    wRadian = w     * math.atan(1.0) / 45.0;

    if param.spiralType == 'log':
	  f = param.a * math.exp(param.k*wRadian)
    else: f = param.a * wRadian

    pos.X = x + f * math.cos(aRadian + wRadian)
    pos.Y = y + f * math.sin(aRadian + wRadian)

    return pos

def createSpiral(param,x,y,angle):

    theCADdy = GetApplic()
    factory       = theCADdy.Factory2D()
    targetManager = theCADdy.TargetManager()
    types         = theCADdy.DataTypes()
    resolution    = theCADdy.ModelManager().GlobalResolution

    width = param.w2 - param.w1;
    iCount = int(width / param.wStep + 0.5) + 1
    if iCount < 0:
        iCount = -iCount
    if iCount < 3:
        iCount = 3

    w = param.w1
    step = width / float(iCount - 1)
    pos1 = getPoint(param,x,y,angle,w)
    w += step
    pos2 = getPoint(param,x,y,angle,w)
    w += step
    pos3 = getPoint(param,x,y,angle,w)
    spline = factory.CreateOp2DSplSeg(pos1,pos2,pos3,True,False)

    i=4
    while i <= iCount:
        w += step
	i += 1
        pos = getPoint(param,x,y,angle,w)
        spline.AddPosition(pos, False)

    AddToParent(spline, targetManager.TargetOp2DParent)

###############################################################################
## The AddIn Server
##
class PySpiral(AddInBase):
    _reg_clsid_  = CLSID
    _reg_progid_ = PROGID

    def __init__(self):
        AddInBase.__init__(self)

    def OnInit(self, theCADdy):
        self.cmdPool = CommandPool(theCADdy)
        
    def OnExit(self, theCADdy):
        self.cmdPool.Delete()

###############################################################################
## The Command Pool 
##
class CommandPool(CmdPoolBase):
    def __init__(self, theCADdy):
        CmdPoolBase.__init__(self, theCADdy)

        newPool = self.AddPool(self.rootPool, "pySpiral")
        newCmd  = self.AddCmd(newPool, "actCreate", constants.CMD_BREAK, CmdActCreate)

    def Delete(self):
        CmdPoolBase.Delete(self)

###############################################################################
## The Command Event Handler
##
class CmdActCreate(CmdEvtHandlerBase):
    """ start action to create and place a Python Spiral
    """
    def __init__(self):
        """Constructor - just pass all parameters to the base class"""
        #set_trace()
        CmdEvtHandlerBase.__init__(self)
        self.myAction = None
        self.param = ParameterSpiral ('log', 10.0, 1.0, 30.0, 450.0, 1.0)

    def OnExecute(self, cmd, data, cmdStr):
        """Envent handler - create an action object."""
        #set_trace()
        uiManager=GetApplic().UIManager()
        frameManager=GetApplic().FrameManager()
        spiralType="log" #frameManager.ScanCmdLine(data,1)

        #uiManager.Information('spiralType = ' + spiralType)

        param = ParameterDialog(self.param)
	if param.valid:
        	self.myAction = ActCreate(cmd, data, cmdStr, param)

###############################################################################
## action ActCreate
##
class ActCreate(ActionBase):
    """create Python Spiral an place geometry
    """
    def __init__(self, cmd, data, cmdStr, param):
        """Constructor
        """
        #set_trace()
        ActionBase.__init__(self, cmd, data, cmdStr)
	self.Start()
        self.frameManager.Prompt.Push("Referenzpunkt festlegen")
        self.pointDefFilter = self.frameManager.UsePointDefFilter(0)
        self.param=param
        self.angle=0.0
        self.frameManager.StatusBlock.ValueAngle = self.angle


    def OnActionDestroy(self):
        """Destructor
        """
        self.pointDefFilter = None
        self.frameManager.StatusBlock.IsUndefinedAngle  = 1
        self.frameManager.Prompt.Pop("")

        return constants.ACTIONEVENT_CONTINUE
        
    def OnActionBreak(self, event):
        """OnActionBreak event handler - break event.
        """
        return constants.ACTIONEVENT_CONTINUE
        
    def OnActionValue(self, event):
        #self.uiManager.Information("OnActionValue Name='" + event.Name +"'")
        if event.Name == 'Angle':
            self.angle = event.Double
            self.frameManager.StatusBlock.ValueAngle = self.angle
        return constants.ACTIONEVENT_STOP

    def OnActionPoint2D(self, event):
        """OnActionPoint2D event handler - point event.
        """
        #set_trace()
        #self.uiManager.Information("OnActionPoint2D")
        x = event.XPosition
        y = event.YPosition
        createSpiral(self.param,x,y,self.angle)

        self.undoRedoManager.SwitchState()
 
        return constants.ACTIONEVENT_CONTINUE
        
    def OnActionDynamic2D(self, event):
        """OnActionDynamic2D event handler - dynamic event.
        """
        return constants.ACTIONEVENT_CONTINUE

### main ######################################################################

def main():
    """Register the AddIn in the Windows registry as an valid COM server and
    start the default command
    """
    RegisterServer(PySpiral)

    theCADdy = GetApplic()
    try:
        theCADdy.Start(PROGID)
    except:
        pass

    theCADdy.FrameManager().SendCommand(".pySpiral.actCreate log" )

if __name__ == '__main__':
    main()
